/// <reference types="node" />
import { IncomingMessage, ServerResponse } from 'http';
import { URL } from 'url';
import { ContentType } from './ContentType';
import { Verb } from './Verb';
/**
 * The Context object for the Handler Pipeline
 */
export declare class Context<DATA extends Record<string, unknown> = {}> {
    /** The IncomingMessage object */
    request: IncomingMessage;
    /** The ServerResponse object */
    response: ServerResponse;
    /** HTTP Verb */
    method: Verb;
    /** URL */
    url: URL;
    /** HTTP Content-Type header */
    contentType: ContentType;
    /** HTTP Accept Header */
    accept: string[];
    data: DATA;
    /** View template */
    template?: string;
    /**
     * The Context constructor
     * @param request - an incoming request
     * @param response - an outgoing response
     */
    constructor(request: IncomingMessage, response: ServerResponse, initialData?: DATA);
}
export declare function getUrl(request: IncomingMessage): URL;
export declare function getMethod(request: IncomingMessage): Verb;
export declare function getContentType(request: IncomingMessage): ContentType;
export declare function getAccept(request: IncomingMessage): string[];
//# sourceMappingURL=Context.d.ts.map