"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAccept = exports.getContentType = exports.getMethod = exports.getUrl = exports.Context = void 0;
const url_1 = require("url");
const Verb_1 = require("./Verb");
/**
 * The Context object for the Handler Pipeline
 */
// eslint-disable-next-line @typescript-eslint/ban-types
class Context {
    /**
     * The Context constructor
     * @param request - an incoming request
     * @param response - an outgoing response
     */
    constructor(request, response, initialData = {}) {
        this.request = request;
        this.response = response;
        this.data = initialData;
        // Method
        this.method = getMethod(request);
        // Url
        this.url = getUrl(request);
        // Content Type
        this.contentType = getContentType(request);
        // Accept Type
        this.accept = getAccept(request);
    }
}
exports.Context = Context;
function getUrl(request) {
    var _a;
    return new url_1.URL((_a = request.url) !== null && _a !== void 0 ? _a : '', `http://${request.headers.host}`);
}
exports.getUrl = getUrl;
function getMethod(request) {
    return Verb_1.getVerb(request.method);
}
exports.getMethod = getMethod;
function getContentType(request) {
    const contentTypeHeader = request.headers['content-type'];
    const contentType = {};
    if (contentTypeHeader) {
        const parts = contentTypeHeader.split('; ');
        contentType.mediaType = parts[0].trim().toLowerCase();
        if (parts.length > 1) {
            const hash = {};
            for (let index = 1, length = parts.length; index < length; index++) {
                const part = parts[index];
                const hashParts = part.split('=');
                if (hashParts.length > 1) {
                    hash[hashParts[0].trim()] = hashParts[1].trim();
                }
            }
            contentType.charset = hash.charset;
            contentType.boundary = hash.boundary;
        }
    }
    return contentType;
}
exports.getContentType = getContentType;
function getAccept(request) {
    const accept = request.headers.accept;
    if (accept) {
        const types = accept.split(',');
        return types.map((type) => {
            const parts = type.split(';');
            return parts[0];
        });
    }
    else {
        return [];
    }
}
exports.getAccept = getAccept;
//# sourceMappingURL=Context.js.map