import { IncomingMessage, ServerResponse } from 'http';
import { Directive, Middleware, MiddlewareContext, MiddlewareReturn, Pipeline } from '@cardboardrobots/pipeline';
import { Context } from '../context';
import { AutoDirective, ErrorDirective, JsonDirective, RawDirective, TextDirective, ViewDirective } from '../directive';
import { LogLevel } from './LogLevel';
export declare type ViewContext<CONTEXT extends Context, VALUE> = CONTEXT & {
    view: ViewDirective<VALUE>;
};
export declare type ErrorContext<CONTEXT extends Context> = CONTEXT & {
    error: Error;
};
export declare class Handler<CONTEXT extends Context = Context, RESULT = void> {
    pipeline: Pipeline<Context, void, CONTEXT, RESULT>;
    errorPipeline: Pipeline<ErrorContext<CONTEXT>, any, any, any>;
    viewPipeline: Pipeline<ViewContext<CONTEXT, RESULT>, RESULT, any, any>;
    logging: LogLevel;
    defaultTemplate: string;
    callback: (request: IncomingMessage, response: ServerResponse) => Promise<void>;
    sendJson<T = RESULT>(context: CONTEXT, directive: JsonDirective<T>): Promise<boolean>;
    sendRaw<T = RESULT>(context: CONTEXT, directive: RawDirective<T>): Promise<boolean>;
    sendText<T = RESULT>(context: CONTEXT, directive: TextDirective<T>): Promise<boolean>;
    sendView(context: CONTEXT, directive: ViewDirective<RESULT>): Promise<boolean>;
    sendAuto(context: CONTEXT, directive: AutoDirective<RESULT>): Promise<void>;
    sendError<T extends Error>(context: CONTEXT, directive: ErrorDirective<T>): Promise<void>;
    send(context: CONTEXT, directive: Directive<RESULT>): Promise<boolean> | Promise<void>;
    log(context: Context, status?: number): void;
    use(middleware: Middleware<CONTEXT, RESULT, RESULT>): this;
    use<NEWDATA extends Record<string, any>, NEWRESULT = RESULT>(middleware: Middleware<CONTEXT & Context<Partial<NEWDATA>>, RESULT, NEWRESULT>): Handler<CONTEXT & Context<NEWDATA>, NEWRESULT>;
    use<MIDDLEWARE extends Middleware<any, any, any>>(middleware: MIDDLEWARE): Handler<CONTEXT & MiddlewareContext<MIDDLEWARE>, MiddlewareReturn<MIDDLEWARE>>;
    useError<NEWRESULT>(middlware: Middleware<ErrorContext<CONTEXT>, Error, NEWRESULT>): Pipeline<ErrorContext<CONTEXT>, any, any, NEWRESULT>;
    useView<NEWRESULT>(middlware: Middleware<ViewContext<CONTEXT, NEWRESULT>, RESULT, NEWRESULT>): Pipeline<ViewContext<CONTEXT, RESULT>, RESULT, any, NEWRESULT>;
    private write;
}
export declare function errorTemplate(error: any): string;
export declare function colorStatus(status: number): string;
//# sourceMappingURL=Handler.d.ts.map