/// <reference types="node" />
import { Server } from 'http';
import { ListenOptions } from 'net';
import { Handler } from '../handler';
export declare class PromiseServer<HANDLER extends Handler<any, any>> extends Server {
    handler: HANDLER;
    constructor(handler: HANDLER);
    /**
     * Opens the `http.Server` to listen on the specified Port.
     * @param port - the Port to open
     * @param hostname - the Hostname
     * @param backlog - the Backlog number
     * @param listeningListener - a callback for the "listening" event
     */
    start(port?: number, hostname?: string, backlog?: number, listeningListener?: () => void): Promise<Server>;
    start(port?: number, hostname?: string, listeningListener?: () => void): Promise<Server>;
    start(port?: number, backlog?: number, listeningListener?: () => void): Promise<Server>;
    start(port?: number, listeningListener?: () => void): Promise<Server>;
    /**
     * Opens the `http.Server` to listen on the specified Port.
     * @param path - the Path to open
     * @param backlog - the Backlog number
     * @param listeningListener - a callback for the "listening" event
     */
    start(path: string, backlog?: number, listeningListener?: () => void): Promise<Server>;
    start(path: string, listeningListener?: () => void): Promise<Server>;
    /**
     * Opens the `http.Server` to listen on the specified Port.
     * @param options - a options object
     * @param listeningListener - a callback for the "listening" event
     */
    start(options: ListenOptions, listeningListener?: () => void): Promise<Server>;
    /**
     * Opens the `http.Server` to listen on the specified Port.
     * @param handle - the Handle to open
     * @param backlog - the Backlog number
     * @param listeningListener - a callback for the "listening" event
     */
    start(handle: any, backlog?: number, listeningListener?: () => void): Promise<Server>;
    start(handle: any, listeningListener?: () => void): Promise<Server>;
    /**
     * Closes the `http.Server`.
     */
    exit(): Promise<void>;
    /**
     * Waits for `SIGINT` or `SIGTERM` signals on the `Process` object.
     */
    wait(): Promise<void>;
}
//# sourceMappingURL=PromiseServer.d.ts.map